searchNodes=[{"doc":"Functions for implementing a parent process.A parent process is a process that manages the lifecycle of its children. Typically the simplest approach is to use higher-level abstractions, such as Parent.Supervisor or Parent.GenServer. The common behaviour for every parent process is implemented in this module, and therefore it is described in this document.OverviewA parent process has the following properties:It traps exits and uses the shutdown: :infinity shutdown strategy.It keeps track of its children.It presents itself to the rest of the OTP as a supervisor, which means that generic code walking the supervision tree, such as OTP release handler, will also iterate the parent's subtree.Before terminating, it stops its children synchronously, in the reverse startup order.You can interact with the parent process from other processes using functions from the Parent.Client module. If you want to manipulate the parent from the inside, you can use the functions from this module.InitializationA parent process has to be initialized using initialize/1. This function takes the following initialization options::max_restarts and :max_seconds - same as with Supervisor, with the same defaults:registry? - If true, the parent will manage its own ETS-based child registry. See the &quot;Child discovery&quot; section for details.When using higher-level abstractions, these options are typically passed throguh start functions, such as Parent.Supervisor.start_link/2.Child specificationChild specification describes how the parent starts and manages a child. This specification is passed to functions such as start_child/2, Parent.Client.start_child/2, or Parent.Supervisor.start_link/2 to start a child process.The specification is a map which is a superset of the Supervisor child specifications. All the fields that are shared with Supervisor have the same effect.It's worth noting that the :id field is optional. If not provided, the child will be anonymous, and you can only manage it via its pid. Therefore, the minimum required child specification is %{start: mfa_or_zero_arity_fun}.Also, just like with Supervisor, you can provide module | {module, arg} when starting a child. See Supervisor.child_spec/1 for details.To modify a child specification, Parent.child_spec/2 can be used.Bound childrenYou can bind the lifecycle of each child to the lifecycles of its older siblings. This is roughly similar to the :rest_for_one supervisor strategy.For example, if you want to start two children, a consumer and a producer, and bind the producer's lifecycle to the consumer, you need the following child specifications:consumer_spec = %{ id: :consumer, # ... } producer_spec = %{ id: :producer, binds_to: [:consumer] }This will make sure that if the consumer stops, the producer is taken down as well.For this to work, you need to start the consumer before the producer. In other words, a child can only be bound to its older siblings.It's worth noting that bindings are transitive. If a child A is bound to the child B, which is in turns bound to child C, then child A also depends on child C. If child C stops, B and A will be stopped to.Shutdown groupsA shutdown group is a mechanism that roughly emulates the :one_for_all supervisor strategy. For example, to set up a two-way lifecycle dependency between the consumer and the producer, we can use the following specifications:consumer_spec = %{ id: :consumer, shutdown_group: :consumer_and_producer # ... } producer_spec = %{ id: :producer, shutdown_group: :consumer_and_producer }In this case, when any child of the group terminates, the other children will be taken down as well. All children belonging to the same shutdown group must use the same :restart and :ephemeral settings.Note that a child can be a member of some shutdown group, and at the same time bound to other older siblings.Lifecycle dependency consequencesA lifecycle dependency means that a child is taken down when its dependency stops. This will happen irrespective of how the child has been stopped. Even if you manually stop the child using functions such as shutdown_child/1 or Parent.Client.shutdown_child/2, the siblings bound to it will be taken down.In general, parent doesn't permit the state which violates the binding settings. If the process A is bound to the process B, parent will not allow A to keep running if B stops.Handling child terminationWhen a child terminates, depending on its :restart and :ephemeral settings, parent will do one of the following things:restart the child (possibly giving up if restart limit has been exceeded)set the child's pid to :undefinedremove the child from its internal structuresThe :restart option controls when a child will be automatically restarted by its parent::permanent - A child is automatically restarted if it stops. This is the default value.:transient - A child is automatically restarted only if it exits abnormally.:temporary - A child is not automatically restarted.The :ephemeral option controls what to do when a non-running child is will not be restarted. Such situation can happen when a temporary child terminates, when a transient child stops normally, or if the child's start function returns :ignore.If the child is not marked as ephemeral (default), parent will keep the child in its internal structures, setting its pid to :undefined. Functions such as Parent.children/0 will include the non-running child in the result, and such child can be restarted using Parent.restart_child/1. This mimics the behaviour of &quot;static supervisors&quot; (i.e. one_for_one, rest_for_one, one_for_all).If the child is marked as ephemeral, parent will remove the child from its internal structures. This mimics the behaviour of DynamicSupervisor.In all of these cases, parent will perform the same action on bound siblings, ignoring their :restart and :ephemeral settings. For example, if a permanent child is restarted, parent will restart all of its bound siblings (including the temporary ones). Likewise, if a temporary child stops, parent will stop all of its bound siblings, including the permanent ones. If an ephemeral child stops, parent will remove all of its bound siblings, including the non-ephemeral ones.If a child is not restarted, its ephemeral bound siblings will be removed. This is the only case where parent honors the :ephemeral status of bound siblings.Restart flowProcess restarts can be triggered by the following situations:a child terminated and it will be restarted due to its :restart settingrestart_child/1 has been invoked (manual restart)return_children/1 has been invoked (returning removed children)In all these situations the flow is the same. Parent will first synchronously stop the bound dependencies of the child (in the reverse startup order). Then it will attempt to restart the child and its siblings. This is done by starting processes synchronously, one by one, in the startup order. If all processes are started successfully, restart has succeeded.If some process fails to start, the parent treats it as a crash. It will take down all the bound siblings, while proceeding to start other children which are not depending on the failed process.Therefore, a restart may partially succeed, with some children not being started. In this case, the parent will retry to restart such children, according to their specification. Temporary children which fail to restart will be considered as stopped, and parent will not attempt to restart them again.A failed attempt to restart a child is considered as a crash and contributes to the restart intensity. Thus, if a child repeatedly fails to restart, the parent will give up at some point, according to restart intensity settings.The restarted children keep their original startup order with respect to non-restarted children. For example, suppose that four children are running: A, B, C, and D, and children B and D are restarted. If the parent process then stops, it will take the children down in the order D, C, B, and A.Maximum restart frequencySimilarly to Supervisor, a parent process keeps track of the amount of restarts, and self-terminates if maximum threshold (defaults to 3 restarts in 5 seconds) is exceeded.In addition, you can provide child specific thresholds by including :max_restarts and :max_seconds options in child specification. Note that :max_restarts can be set to :infinity (both for the parent and each child). This can be useful if you want to disable the parent global limit, and use child-specific limits.Finally, it's worth noting that if termination of one child causes the restart of multiple children, parent will treat this as a single restart event when calculating the restart frequency and considering possible self-termination.Child timeoutYou can optionally include the :timeout option in the child specification to ask the parent to terminate the child if it doesn't stop in the given time. The child will be terminated according to its shutdown specification. In the case of non-forceful termination, the :timeout exit signal will be used.A non-temporary child which timeouts will be restarted.Child discoveryChildren can be discovered by other processes using functions such as Parent.Client.child_pid/2, or Parent.Client.children/1. By default, these functions will perform a synchronous call into the parent process. This should work fine as long as the parent is not pressured by various events, such as frequent children stopping and starting, or some other custom logic.In such cases you can consider setting the registry? option to true when initializing the parent process. When this option is set, parent will create an ETS table which will be used by the discovery functions.In addition, parent supports maintaining the child-specific meta information. You can set this information by providing the :meta field in the child specification, update it through functions such as update_child_meta/2 or Parent.Client.update_child_meta/3, and query it through Parent.Client.child_meta/2.Building custom parent processesIf available parent behaviours don't fit your purposes, you can consider building your own behaviour or a concrete parent process. In this case, the functions of this module will provide the necessary plumbing.The basic idea is presented in the following sketch:defp init_process do Parent.initialize(parent_opts) start_some_children() loop() end defp loop() do receive do msg -&gt; case Parent.handle_message(msg) do # parent handled the message :ignore -&gt; loop() # parent handled the message and returned some useful information {:stopped_children, stopped_children} -&gt; handle_stopped_children(stopped_children) # not a parent message nil -&gt; custom_handle_message(msg) end end endMore specifically, to build a parent process you need to do the following:Invoke initialize/0 when the process is started.Use functions such as start_child/2 to work with child processes.When a message is received, invoke handle_message/1 before handling the message yourself.If you receive a shutdown exit message from your parent, stop the process.Before terminating, invoke shutdown_all/1 to stop all the children.Use :infinity as the shutdown strategy for the parent process, and :supervisor for its type.If the process is a GenServer, handle supervisor calls (see supervisor_which_children/0 and supervisor_count_children/0).Implement format_status/2 (see Parent.GenServer for details) where applicable.If the parent process is powered by a non-interactive code (e.g. Task), make sure to receive messages sent to that process, and handle them properly (see points 3 and 4).You can take a look at the code of Parent.GenServer for specific details.","ref":"Parent.html","title":"Parent","type":"module"},{"doc":"Returns true if the child process is still running, false otherwise.Note that this function might return true even if the child has terminated. This can happen if the corresponding :EXIT message still hasn't been processed, and also if a non-ephemeral child is not running.","ref":"Parent.html#child?/1","title":"Parent.child?/1","type":"function"},{"doc":"Returns the id of a child process with the given pid.","ref":"Parent.html#child_id/1","title":"Parent.child_id/1","type":"function"},{"doc":"Returns the meta associated with the given child id.","ref":"Parent.html#child_meta/1","title":"Parent.child_meta/1","type":"function"},{"doc":"Returns the pid of a child process with the given id.","ref":"Parent.html#child_pid/1","title":"Parent.child_pid/1","type":"function"},{"doc":"Builds and overrides a child specificationThis operation is similar to Supervisor.child_spec/1","ref":"Parent.html#child_spec/2","title":"Parent.child_spec/2","type":"function"},{"doc":"Returns the list of running child processes in the startup order.","ref":"Parent.html#children/0","title":"Parent.children/0","type":"function"},{"doc":"Should be invoked by the parent process for each incoming message.If the given message is not handled, this function returns nil. In such cases, the client code should perform standard message handling. Otherwise, the message has been handled by the parent, and the client code shouldn't treat this message as a standard message (e.g. by calling handle_info of the callback module).If :ignore is returned, the message has been processed, and the client code should ignore it. Finally, if the return value is {:stopped_children, info}, it indicates that some ephemeral processes have stopped and have been removed from parent. A client may do some extra processing in this case.Note that you don't need to invoke this function in a Parent.GenServer callback module.","ref":"Parent.html#handle_message/1","title":"Parent.handle_message/1","type":"function"},{"doc":"Initializes the state of the parent process.This function should be invoked once inside the parent process before other functions from this module are used. If a parent behaviour, such as Parent.GenServer, is used, this function must not be invoked.","ref":"Parent.html#initialize/1","title":"Parent.initialize/1","type":"function"},{"doc":"Returns true if the parent state is initialized.","ref":"Parent.html#initialized?/0","title":"Parent.initialized?/0","type":"function"},{"doc":"Returns the count of children.","ref":"Parent.html#num_children/0","title":"Parent.num_children/0","type":"function"},{"doc":"","ref":"Parent.html#parent_spec/1","title":"Parent.parent_spec/1","type":"function"},{"doc":"Restarts the child and its siblings.See &quot;Restart flow&quot; for details on restarting procedure.","ref":"Parent.html#restart_child/1","title":"Parent.restart_child/1","type":"function"},{"doc":"Starts new instances of stopped children.This function can be invoked to return stopped children back to the parent. Essentially, this function works the same as automatic restarts.The stopped_children information is obtained via functions such as shutdown_child/1 or shutdown_all/1. In addition, Parent will provide this info via handle_message/1 if an ephemeral child stops and is not restarted.","ref":"Parent.html#return_children/1","title":"Parent.return_children/1","type":"function"},{"doc":"Terminates all running child processes.Children are terminated synchronously, in the reverse order from the order they have been started in. All corresponding :EXIT messages will be pulled from the mailbox.","ref":"Parent.html#shutdown_all/1","title":"Parent.shutdown_all/1","type":"function"},{"doc":"Shuts down the child and all siblings depending on it, and removes them from the parent state.This function will also shut down all siblings directly and transitively bound to the given child. The function will wait for the child to terminate, and pull the :EXIT message from the mailbox.All terminated children are removed from the parent state. The stopped_children structure describes all of these children, and can be used with return_children/1 to manually restart these processes.","ref":"Parent.html#shutdown_child/1","title":"Parent.shutdown_child/1","type":"function"},{"doc":"Synchronously starts all children.If some child fails to start, all of the children will be taken down and the parent process will exit.","ref":"Parent.html#start_all_children!/1","title":"Parent.start_all_children!/1","type":"function"},{"doc":"Starts the child described by the specification.","ref":"Parent.html#start_child/2","title":"Parent.start_child/2","type":"function"},{"doc":"Should be invoked by the behaviour when handling :count_children GenServer call.See supervisor_which_children/0 for details.","ref":"Parent.html#supervisor_count_children/0","title":"Parent.supervisor_count_children/0","type":"function"},{"doc":"Should be invoked by the behaviour when handling :get_childspec GenServer call.See :supervisor.get_childspec/2 for details.","ref":"Parent.html#supervisor_get_childspec/1","title":"Parent.supervisor_get_childspec/1","type":"function"},{"doc":"Should be invoked by the behaviour when handling :which_children GenServer call.You only need to invoke this function if you're implementing a parent process using a behaviour which forwards GenServer call messages to the handle_call callback. In such cases you need to respond to the client with the result of this function. Note that parent behaviours such as Parent.GenServer will do this automatically.If no translation of GenServer messages is taking place, i.e. if you're handling all messages in their original shape, this function will be invoked through handle_message/1.","ref":"Parent.html#supervisor_which_children/0","title":"Parent.supervisor_which_children/0","type":"function"},{"doc":"Updates the meta of the given child process.","ref":"Parent.html#update_child_meta/2","title":"Parent.update_child_meta/2","type":"function"},{"doc":"","ref":"Parent.html#t:child/0","title":"Parent.child/0","type":"type"},{"doc":"","ref":"Parent.html#t:child_id/0","title":"Parent.child_id/0","type":"type"},{"doc":"","ref":"Parent.html#t:child_meta/0","title":"Parent.child_meta/0","type":"type"},{"doc":"","ref":"Parent.html#t:child_ref/0","title":"Parent.child_ref/0","type":"type"},{"doc":"","ref":"Parent.html#t:child_spec/0","title":"Parent.child_spec/0","type":"type"},{"doc":"","ref":"Parent.html#t:handle_message_response/0","title":"Parent.handle_message_response/0","type":"type"},{"doc":"","ref":"Parent.html#t:on_start_child/0","title":"Parent.on_start_child/0","type":"type"},{"doc":"","ref":"Parent.html#t:option/0","title":"Parent.option/0","type":"type"},{"doc":"","ref":"Parent.html#t:opts/0","title":"Parent.opts/0","type":"type"},{"doc":"","ref":"Parent.html#t:shutdown/0","title":"Parent.shutdown/0","type":"type"},{"doc":"","ref":"Parent.html#t:shutdown_group/0","title":"Parent.shutdown_group/0","type":"type"},{"doc":"","ref":"Parent.html#t:start/0","title":"Parent.start/0","type":"type"},{"doc":"","ref":"Parent.html#t:start_error/0","title":"Parent.start_error/0","type":"type"},{"doc":"","ref":"Parent.html#t:start_spec/0","title":"Parent.start_spec/0","type":"type"},{"doc":"","ref":"Parent.html#t:stopped_children/0","title":"Parent.stopped_children/0","type":"type"},{"doc":"Functions for interacting with parent's children from other processes.All of these functions issue a call to the parent process. Therefore, they can't be used from inside the parent process. Use functions from the Parent module instead to interact with the children from within the process.Likewise these functions can't be invoked inside the child process during its initialization. Defer interacting with the parent to GenServer.handle_continue/2, or if you're using another behaviour which doesn't support such callback, send yourself a message to safely do the post-init interaction with the parent.If parent is configured with the registry?: true option, some query functions, such as child_pid/2 will perform an ETS lookup instead of issuing a call, so the caveats above won't apply.","ref":"Parent.Client.html","title":"Parent.Client","type":"module"},{"doc":"Client interface to Parent.child_meta/1.If the parent is a registry, the result will be obtained from the ETS table.","ref":"Parent.Client.html#child_meta/2","title":"Parent.Client.child_meta/2","type":"function"},{"doc":"Client interface to Parent.child_pid/1.If the parent is a registry, the result will be obtained from the ETS table.","ref":"Parent.Client.html#child_pid/2","title":"Parent.Client.child_pid/2","type":"function"},{"doc":"Client interface to Parent.children/0.If the parent is a registry, the result will be obtained from the ETS table.","ref":"Parent.Client.html#children/1","title":"Parent.Client.children/1","type":"function"},{"doc":"Client interface to Parent.restart_child/1.","ref":"Parent.Client.html#restart_child/2","title":"Parent.Client.restart_child/2","type":"function"},{"doc":"Client interface to Parent.return_children/1.","ref":"Parent.Client.html#return_children/2","title":"Parent.Client.return_children/2","type":"function"},{"doc":"Client interface to Parent.shutdown_all/1.","ref":"Parent.Client.html#shutdown_all/2","title":"Parent.Client.shutdown_all/2","type":"function"},{"doc":"Client interface to Parent.shutdown_child/1.","ref":"Parent.Client.html#shutdown_child/2","title":"Parent.Client.shutdown_child/2","type":"function"},{"doc":"Client interface to Parent.start_child/2.","ref":"Parent.Client.html#start_child/3","title":"Parent.Client.start_child/3","type":"function"},{"doc":"Client interface to Parent.update_child_meta/2.","ref":"Parent.Client.html#update_child_meta/3","title":"Parent.Client.update_child_meta/3","type":"function"},{"doc":"GenServer with parenting capabilities powered by Parent.This behaviour can be useful in situations where Parent.Supervisor won't suffice.ExampleThe following example is roughly similar to a standard callback-based Supervisor:defmodule MyApp.Supervisor do # Automatically defines child_spec/1 use Parent.GenServer def start_link(init_arg), do: Parent.GenServer.start_link(__MODULE__, init_arg, name: __MODULE__) @impl GenServer def init(_init_arg) do Parent.start_all_children!(children) {:ok, initial_state} end endThe expression use Parent.GenServer will also inject use GenServer into your code. Your parent process is a GenServer, and this behaviour doesn't try to hide it. Except when starting the process, you work with the parent exactly as you work with any GenServer, using the same functions, such as GenServer.call/3, and providing the same callbacks, such as init/1, or handle_call/3.Interacting with the parent from the outsideYou can issue regular GenServer calls and casts, and send messages to the parent, which can be handled by corresponding GenServer callbacks. In addition, you can use functions from the Parent.Client module to manipulate or query the parent state from other processes. As a good practice, it's advised to wrap such invocations in the module which implements Parent.GenServer.Interacting with children inside the parentFrom within the parent process, you can interact with the child processes using functions from the Parent module. All child processes should be started using Parent functions, such as Parent.start_child/2, because otherwise Parent won't be aware of these processes and won't be able to fulfill its guarantees.Note that you can start children from any callback, not just during init/1. In addition, you don't need to start all children at once. Therefore, Parent.GenServer can prove useful when you need to make some runtime decisions: {:ok, child1} = Parent.start_child(child1_spec) if some_condition_met?, do: Parent.start_child(child2_spec) Parent.start_child(child3_spec)However, bear in mind that this code won't be executed again if the processes are restarted.Handling child terminationIf a child process terminates and isn't restarted, the handle_stopped_children/2 callback is invoked. The default implementation does nothing.The following example uses handle_stopped_children/2 to start a child task and report if it it crashes: defmodule MyJob do use Parent.GenServer, restart: :temporary def start_link(arg), do: Parent.GenServer.start_link(__MODULE__, arg) @impl GenServer def init(_) do {:ok, _} = Parent.start_child(%{ id: :job, start: {Task, :start_link, [fn -&gt; job(arg) end]}, restart: :temporary, # handle_stopped_children won&#39;t be invoked without this ephemeral?: true }) {:ok, nil} end @impl Parent.GenServer def handle_stopped_children(%{job: info}, state) do if info.reason != :normal do # report job failure end {:stop, reason, state} end endhandle_stopped_children can be useful to implement arbitrary custom behaviour, such as restarting after a delay, and using incremental backoff periods between two consecutive starts.For example, this is how you could introduce a delay between two consecutive starts:def handle_stopped_children(stopped_children, state) do Process.send_after(self, {:restart, stopped_children}, delay) {:noreply, state} end def handle_info({:restart, stopped_children}, state) do Parent.return_children(stopped_children) {:noreply, state} endKeep in mind that handle_stopped_children is only invoked if the child crashed on its own, and if it's not going to be restarted.If the child was explicitly stopped via a Parent function, such as Parent.shutdown_child/1, this callback will not be invoked. The same holds for Parent.Client functions. If you want to unconditionally react to a termination of a child process, setup a monitor with Process.monitor and add a corresponding handle_info clause.If the child was taken down because its lifecycle is bound to some other process, the corresponding handle_stopped_children won't be invoked. For example, if process A is bound to process B, and process B crashes, only one handle_stopped_children will be invoked (for the crash of process B). However, the corresponding info will contain the list of all associated siblings that have been taken down, and stopped_children will include information necessary to restart all of these siblings. Refer to Parent documentation for details on lifecycles binding.Parent terminationThe behaviour takes down the child processes before it terminates, to ensure that no child process is running after the parent has terminated. The children are terminated synchronously, one by one, in the reverse start order.The termination of the children is done after the terminate/1 callback returns. Therefore in terminate/1 the child processes are still running, and you can interact with them, and even start additional children.CaveatsLike any other Parent-based process, Parent.GenServer traps exits and uses the :infinity shutdown strategy. As a result, a parent process which blocks for a long time (e.g. because its communicating with a remote service) won't be able to handle child termination, and your fault-tolerance might be badly affected. In addition, a blocking parent might completely paralyze the system (or a subtree) shutdown. Setting a shutdown strategy to a finite time is a hacky workaround that will lead to lingering orphan processes, and might cause some strange race conditions which will be very hard to debug.Therefore, be wary of having too much logic inside a parent process. Try to push as much responsibilities as possible to other processes, such as children or siblings, and use parent only for coordination and reporting tasks.Finally, since parent trap exits, it's possible to receive an occasional stray :EXIT message if the child crashes during its initialization.By default use Parent.GenServer receives such messages and ignores them. If you're implementing your own handle_info, make sure to include a clause for :EXIT messages: def handle_info({:EXIT, _pid, _reason}, state), do: {:noreply, state}","ref":"Parent.GenServer.html","title":"Parent.GenServer","type":"behaviour"},{"doc":"Invoked when some children have terminated.The info map will contain all the children which have been stopped together. For example, if child A is bound to child B, and child B terminates, parent will also terminate the child A. In this case, handle_stopped_children is invoked only once, with the info map containing entries for both children.This callback will not be invoked in the following cases:a child is terminated by invoking Parent functions such as Parent.shutdown_child/1a child is restarteda child is not ephemeral (see &quot;Ephemeral children&quot; in Parent for details)","ref":"Parent.GenServer.html#c:handle_stopped_children/2","title":"Parent.GenServer.handle_stopped_children/2","type":"callback"},{"doc":"Starts the parent process.","ref":"Parent.GenServer.html#start_link/3","title":"Parent.GenServer.start_link/3","type":"function"},{"doc":"","ref":"Parent.GenServer.html#t:options/0","title":"Parent.GenServer.options/0","type":"type"},{"doc":"","ref":"Parent.GenServer.html#t:state/0","title":"Parent.GenServer.state/0","type":"type"},{"doc":"Supervisor of child processes.This module works similarly to callbackless supervisors started with Supervisor.start_link/2. To start a supervisor and some children you can do the following:Parent.Supervisor.start_link([ child_spec1, child_spec2, # ... ])To install a parent supervisor in the supervision tree you can provide child specification in the shape of {Parent.Supervisor, {children, parent_options}}.To build a dedicate supervisor module you can do:defmodule MySupervisor do use Parent.Supervisor def start_link({children, options}), do: Parent.Supervisor.start_link(children, options) # ... endAnd now, you can install this supervisor in the supervision tree by passing {MySupervisor, {child_specs, parent_options}} as the child specification to the parent.You can interact with the running supervisor using functions from the Parent.Client module. Refer to the Parent module for detailed explanation of child specifications, parent options, and behaviour of parent processes.In case you need more flexibility, take a look at Parent.GenServer.","ref":"Parent.Supervisor.html","title":"Parent.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"Parent.Supervisor.html#child_spec/1","title":"Parent.Supervisor.child_spec/1","type":"function"},{"doc":"Starts the parent process.This function returns only after all the children have been started. If a child fails to start, the parent process will terminate all successfully started children, and then itself.","ref":"Parent.Supervisor.html#start_link/2","title":"Parent.Supervisor.start_link/2","type":"function"},{"doc":"Periodic job execution.Quick startIt is recommended (but not required) to implement the job in a dedicated module. For example:defmodule SomeCleanup do def child_spec(_arg) do Periodic.child_spec( id: __MODULE__, run: &amp;run/0, every: :timer.hours(1) ) end defp run(), do: # ... endWith such module implemented, you can place the job somewhere in the supervision tree:Supervisor.start_link( [ SomeCleanup, # ... ], # ... )You can of course start multiple periodic jobs in the system, and they don't have to be the children of the same supervisor. You're advised to place the job in the proper part of the supervision tree. For example, a database cleanup job should share the ancestor with the repo, while a job working with Phoenix channels should share the ancestor with the endpoint.As mentioned, you don't need to create a dedicated module to run a job. It's also possible to provide {Periodic, opts} in the supervisor child list. Finally, if you need more runtime flexibility, you can also start the job with start_link/1.Process structureThe process started with start_link is called the scheduler. This is the process which regularly &quot;ticks&quot; in the given interval and executes the job. The job is executed in a separate one-off process, which is the child of the scheduler. When the job is done, the job process stops. Therefore, each job instance is running in a separate process.Depending on the overlapping mode (see the :on_overlap option), it can happen that multiple instances of the same job are running simultaneously.Options:run (required) - Zero arity function or MFA invoked to run the job. This function is invoked in a separate one-off process which is a child of the scheduler.:every (required) - Time in milliseconds between two consecutive job executions (see :delay_mode option for details).:initial_delay - Time in milliseconds before the first execution of the job. If not provided, the default value of :every is used. In other words, the first execution will by default take place after the :initial_delay interval has passed.:delay_mode - Controls how the :every interval is interpreted. Following options are possible::regular (default) - :every represents the time between two consecutive starts:shifted - :every represents the time between the termination of the previous and the start of the next instance.See the &quot;Delay mode&quot; section for more details.:when - Function which acts as an additional runtime guard to decide if the job will be started. This can be useful for implementing fixed scheduled jobs. See the &quot;Fixed scheduling&quot; section for details.:on_overlap - Defines the desired behaviour when the job is about to be started while the previous instance is still running.:run (default) - always start the new job:ignore - don't start the new job if the previous instance is still running:stop_previous - stop the previous instance before starting the new one:timeout - Defines the maximum running time of the job. If the job doesn't finish in the given time, it is terminated according to its shutdown specification. Defaults to :infinity.:job_shutdown - Shutdown value of the job process. See the &quot;Shutdown&quot; section for details.:id - Supervisor child id of the scheduler process. Defaults to Periodic. If you plan on running multiple periodic jobs under the same supervisor, make sure that they have different id values.:name - Registered name of the scheduler process. If not provided, the process will not be registered.:telemetry_id - Id used in telemetry event names. See the &quot;Telemetry&quot; section for more details. If not provided, telemetry events won't be emitted.:mode - When set to :manual, the jobs won't be started automatically. Instead you have to manually send tick signals to the scheduler. This should be used only in :test mix env. See the &quot;Testing&quot; section for details.Delay modeIn the :regular mode (which is the default), the interval indicates time between two consecutive starts. This mode is typically useful if you want to maintain a stable execution rate (the number of executions per some time period). It is also a better choice if you're implementing fixed scheduling, as advised in the &quot;Fixed scheduling&quot; section.In the :shifted mode the interval represents the pause time between the end of the job and the start of the next one. This mode is likely a better choice if you want to have a fixed &quot;cool off&quot; period between two consecutive executions, to reduce the load on the system.Internally, Periodic relies on Erlang's monotonic time, which improves rate stability regardless of system time changes (see Time correction). Consider using the &quot;Multi-time warp mode&quot; (see here) to further improve rate stability in the situations when system time changes.In general, the overhead introduced by Periodic as well as job processing will be compensated, and you can usually expect stable intervals with very small variations (typically in sub milliseconds range), and no steady shift over time. However, in some cases, for example when the system is overloaded, the variations might be more significant.In the :shifted mode the job duration will affect the execution of the next job. In addition, Periodic will induce a slight (usually less than 100 microseconds), but a steady skew, due to its own internal processing.ShutdownTo stop the scheduler, you need to ask its parent supervisor to stop the scheduler using Supervisor.terminate_child.The scheduler process acts as a supervisor, and so it has the same shutdown behaviour. When ordered to terminate by its parent, the scheduler will stop currently running job instances according to the :job_shutdown configuration.The default behaviour is to wait 5 seconds for the job to finish. However, in order for this waiting to actually happen, you need to invoke Process.flag(:trap_exit, true) from the run function.You can change the waiting time with the :job_shutdown option, which has the same semantics as in Supervisor. See [corresponding Supervisor documentation] (https://hexdocs.pm/elixir/Supervisor.html#module-shutdown-values-shutdown) for details.Fixed schedulingPeriodic doesn't have explicit support for scheduling jobs at some particular time (e.g. every day at midnight). However, you can implement this on top of the provided functionality using the :when optiondefmodule SomeCleanup do def child_spec(_arg) do Periodic.child_spec( # check every minute if we need to run the cleanup every: :timer.minutes(1), # start the job only if it&#39;s midnight when: fn -&gt; match?(%Time{hour: 0, minute: 0}, Time.utc_now()) end, # ... ) end # ... endNote that the execution guarantees here are &quot;at most once&quot;. If the system is down at the scheduled time, the job won't be executed. Stronger guarantees can be obtained by basing the conditional logic on some persistence mechanism.Note that the :when guard is executed in the scheduler process. If the guard execution time is larger than the ticking period, time drifts will occur.TelemetryThe scheduler optionally emits telemetry events. To configure telemetry you need to provide the :telemetry_id option. For example:Periodic.start_link(telemetry_id: :db_cleanup, ...)This will emit various events in the shape of [Periodic, telemetry_id, event]. Currently supported events are::started - a new job instance is started:finished - job instance has finished or crashed (see related metadata for the reason):skipped - new instance hasn't been started because the previous one is still running:stopped_previous - previous instance has been stopped because the new one is about to be startedTo consume the desired events, install the corresponding telemetry handler.LoggingBasic logger is provided in Periodic.Logger. To use it, the scheduler needs to be started with the :telemetry_id option.To install logger handlers, you can invoke Periodic.Logger.install(telemetry_id). This function should be invoked only once per each scheduler during the system lifetime, preferably before the scheduler is started. A convenient place to do it is your application start callback.TestingThe scheduler can be deterministically tested by setting the :mode option to :manual. In this mode, the scheduler won't tick on its own, and so it won't start any jobs unless instructed to by the client code.The :mode should be set to :manual only in test mix environment. Here's a simple approach which doesn't require app env and config files:defmodule MyPeriodicJob do @mode if Mix.env() != :test, do: :auto, else: :manual def child_spec(_arg) do Periodic.child_spec( mode: @mode, name: __MODULE__, # ... ) end # ... endOf course, you can alternatively use app env or any other approach you prefer. Just make sure to set the mode to manual only in test env.Notice that we're also setting the registered name and telemetry id. We'll need both to interact with the schedulerWith such setup in place, the general shape of the periodic job test would look like this:def MyPeriodicJobTest do use ExUnit.Case, async: true require Periodic.Test test &quot;my periodic job&quot; do bring_the_system_into_the_desired_state() # tick the scheduler assert Periodic.Test.sync_tick(MyPeriodicJob) == {:ok, :normal} verify_side_effect_of_the_job() end endNote that this won't suffice for fixed schedules. Consider again the cleanup job which runs at midnight:defmodule SomeCleanup do def child_spec(_arg) do Periodic.child_spec( every: :timer.minutes(1), when: fn -&gt; match?(%Time{hour: 0, minute: 0}, Time.utc_now()) end, # ... ) end # ... endManually ticking won't start the job, unless the test is running exactly at midnight. To make this module testable, you need to use a different implementation of :when in test environment:defmodule SomeCleanup do def child_spec(_arg) do Periodic.child_spec( every: :timer.minutes(1), when: &amp;should_run?/0 # ... ) end if Mix.env() != :test do defp should_run?(), do: match?(%Time{hour: 0, minute: 0}, Time.utc_now()) else defp should_run?(), do: true end # ... endComparison to other schedulersThere are various other abstractions for running periodic jobs in BEAM, such as:the built-in :timer module from Erlang stdliberlcronquantumObanCompared to :timer, Periodic offers some additional features, such as overlap handling, distributed scheduling, and telemetry support.Compared to most other third party libraries, Periodic will likely provide much less features out of the box. So in some situations, such as database persistence or back-pressure, you might need to invest more effort with Periodic. On the plus side Periodic should be simpler to use in typical scenarios, and much easier to reason about, while still providing enough flexibility to handle arbitrarily complex scenarios.For a more detailed discussion, see this blog post.","ref":"Periodic.html","title":"Periodic","type":"module"},{"doc":"Builds a child specification for starting the periodic executor.","ref":"Periodic.html#child_spec/1","title":"Periodic.child_spec/1","type":"function"},{"doc":"Starts the periodic executor.","ref":"Periodic.html#start_link/1","title":"Periodic.start_link/1","type":"function"},{"doc":"","ref":"Periodic.html#t:opts/0","title":"Periodic.opts/0","type":"type"},{"doc":"Telemetry handler that support basic logging of periodic scheduler events.","ref":"Periodic.Logger.html","title":"Periodic.Logger","type":"module"},{"doc":"Installs telemetry handlers for the given scheduler.","ref":"Periodic.Logger.html#install/1","title":"Periodic.Logger.install/1","type":"function"},{"doc":"Helpers for testing a periodic job.See the &quot;Testing&quot; section in Periodic documentation for details.","ref":"Periodic.Test.html","title":"Periodic.Test","type":"module"},{"doc":"Waits for the given telemetry event.","ref":"Periodic.Test.html#assert_periodic_event/4","title":"Periodic.Test.assert_periodic_event/4","type":"macro"},{"doc":"Subscribes to telemetry events of the given scheduler.","ref":"Periodic.Test.html#observe/1","title":"Periodic.Test.observe/1","type":"function"},{"doc":"Asserts that the given telemetry event won't be emitted.","ref":"Periodic.Test.html#refute_periodic_event/4","title":"Periodic.Test.refute_periodic_event/4","type":"macro"},{"doc":"Sends a tick signal to the given scheduler and waits for the job to finish.The function returns the job exit reason, or error if the job hasn't been started.","ref":"Periodic.Test.html#sync_tick/2","title":"Periodic.Test.sync_tick/2","type":"function"},{"doc":"Sends a tick signal to the given scheduler.This function returns after the tick signal has been sent, and the job has been started. However, the function doesn't wait for the job to finish. If you want complete synchronism, use sync_tick/2","ref":"Periodic.Test.html#tick/1","title":"Periodic.Test.tick/1","type":"function"},{"doc":"Parent Support for custom parenting of processes. See docs for reference.Parent is a toolkit for building processes which parent other children and manage their life cycles. The library provides features similar to Supervisor, such as the support for automatic restarts and failure escalation (maximum restart intensity), with some additional benefits that can help flattening the supervision tree, reduce the amount of custom process monitors, and simplify the process structure. The most important differences from Supervisor are:No supervision strategies (one_for_one, rest_for_one, etc). Instead, Parent uses bindings and shutdown groups to achieve the similar behaviour.No distinction between static and dynamic supervisors. Instead, a per-child option called :ephemeral? is used to achieve dynamic behaviour.Basic registry-like capabilities for simple children discovery baked-in directly into parent.Exposed lower level plumbing modules, such as Parent.GenServer and Parent, which can be used to build custom parent processes (i.e. supervisors with custom logic).","ref":"readme.html","title":"Parent","type":"extras"},{"doc":"Basic supervisorParent.Supervisor.start_link( # child spec is a superset of supervisor child specification child_specs, # parent options, note that there&#39;s no `:strategy` max_restarts: 3, max_seconds: 5, # std. Supervisor/GenServer options name: __MODULE__ )Binding lifecyclesParent.Supervisor.start_link( [ Parent.child_spec(Child1), Parent.child_spec(Child2, binds_to: [Child1]), Parent.child_spec(Child3, binds_to: [Child1]), Parent.child_spec(Child4, shutdown_group: :children4_to_6), Parent.child_spec(Child5, shutdown_group: :children4_to_6), Parent.child_spec(Child6, shutdown_group: :children4_to_6), Parent.child_spec(Child7, binds_to: [Child1]), ] )if Child1 is restarted, Child2, Child3, and Child7 will be restarted tooif Child2, Child3, or Child7 is restarted, nothing else is restartedif any of Child4, Child5, or Child6 is restarted, all other processes from the shutdown group are restarted tooDiscovering siblings during startupParent.Supervisor.start_link( [ Parent.child_spec(Child1), Parent.child_spec(Child2, binds_to: [Child1]), # ... ] ) defmodule Child2 do def start_link do # can be safely invoked inside the parent process child1 = Parent.child_pid(:child1) # ... end endPausing and resuming a part of the system# stops child1 and all children depending on it, removing it from the parent stopped_children = Parent.Client.shutdown_child(some_parent, :child1) # ... # returns all stopped children back to the parent Parent.Client.return_children(some_parent, stopped_children)Dynamic supervisor with anonymous childrenParent.Supervisor.start_link([]) # set `ephemeral?: true` for dynamic children if child is temporary/transient {:ok, pid1} = Parent.Client.start_child(MySup, Parent.child_spec(Child, id: nil, ephemeral?: true)) {:ok, pid2} = Parent.Client.start_child(MySup, Parent.child_spec(Child, id: nil, ephemeral?: true)) # ... Parent.Client.shutdown_child(MySup, pid1) Parent.Client.restart_child(MySup, pid2)Dynamic supervisor with child discoveryParent.Supervisor.start_link([], name: MySup) # meta is an optional value associated with the child Parent.Client.start_child(MySup, Parent.child_spec(Child, id: id1, ephemeral?: true, meta: some_meta)) Parent.Client.start_child(MySup, Parent.child_spec(Child, id: id2, ephemeral?: true, meta: another_meta)) # ... # synchronous calls into the parent process pid = Parent.Client.child_pid(MySup, id1) meta = Parent.Client.child_meta(MySub, id1) all_children = Parent.Client.children(MySup)Optional ETS-powered registry:Parent.Supervisor.start_link([], registry?: true) # start some children # ETS lookup, no call into parent involved Parent.Client.child_pid(my_sup, id1) Parent.Client.children(my_sup)Per-child max restart frequencyParent.Supervisor.start_link( [ Parent.child_spec(Child1, max_restarts: 10, max_seconds: 10), Parent.child_spec(Child2, max_restarts: 3, max_seconds: 5) ], # Per-parent max restart frequency can be disabled, or a parent-wide limit can be used. In the # former case make sure that this limit is higher than the limit of any child. max_restarts: :infinity )Module-based supervisordefmodule MySup do use Parent.GenServer def start_link(init_arg), do: Parent.GenServer.start_link(__MODULE__, init_arg, name: __MODULE__) @impl GenServer def init(_init_arg) do Parent.start_all_children!(children) {:ok, initial_state} end endRestarting with a delaydefmodule MySup do use Parent.GenServer def start_link(init_arg), do: Parent.GenServer.start_link(__MODULE__, init_arg, name: __MODULE__) @impl GenServer def init(_init_arg) do # Make sure that children are temporary and ephemeral b/c otherwise `handle_stopped_children/2` # won&#39;t be invoked. Parent.start_all_children!(children) {:ok, initial_state} end @impl Parent.GenServer def handle_stopped_children(stopped_children, state) do # invoked when a child stops and is not restarted Process.send_after(self, {:restart, stopped_children}, delay) {:noreply, state} end def handle_info({:restart, stopped_children}, state) do # Returns the child to the parent preserving its place according to startup order and bumping # its restart count. This is basically a manual restart. Parent.return_children(stopped_children) {:noreply, state} end endStarting additional children after a child stopsdefmodule MySup do use Parent.GenServer def start_link(init_arg), do: Parent.GenServer.start_link(__MODULE__, init_arg, name: __MODULE__) @impl GenServer def init(_init_arg) do Parent.start_child(first_child_spec) {:ok, initial_state} end @impl Parent.GenServer def handle_stopped_children(%{child1: info}, state) do Parent.start_child(other_children) {:noreply, state} end def handle_stopped_children(_other, state), do: {:noreply, state} endBuilding a custom parent process or behaviour from scratchdefp init_process do Parent.initialize(parent_opts) start_some_children() loop() end defp loop() do receive do msg -&gt; case Parent.handle_message(msg) do # parent handled the message :ignore -&gt; loop() # parent handled the message and returned some useful information {:stopped_children, stopped_children} -&gt; handle_stopped_children(stopped_children) # not a parent message nil -&gt; custom_handle_message(msg) end end end","ref":"readme.html#examples","title":"Parent - Examples","type":"extras"},{"doc":"This library has seen production usage in a couple of different projects. However, features such as automatic restarts and ETS registry are pretty fresh (aded in late 2020) and so they haven't seen any serious production testing yet.Based on a very quick &amp; shallow test, Parent is about 3x slower and consumes about 2x more memory than DynamicSupervisor.The API is prone to significant changes.Compared to supervisor crash reports, the error logging is very basic and probably not sufficient.","ref":"readme.html#status","title":"Parent - Status","type":"extras"},{"doc":"MIT","ref":"readme.html#license","title":"Parent - License","type":"extras"}]